/*
 * This file is part of Danbooru Client.
 * Copyright 2013  Luca Beltrame <lbeltrame@kde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License or (at your option) version 3 or any later version
 * accepted by the membership of KDE e.V. (or its successor approved
 * by the membership of KDE e.V.), which shall act as a proxy
 * defined in Section 14 of version 3 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef DANBOORUPOSTDELEGATE_H
#define DANBOORUPOSTDELEGATE_H

/**
 * @brief This file includes custom delegates to represent DanbooruItems.
 * @file danboorupostdelegate.h
 *
 **/

#include <QStyledItemDelegate>

#include <KUrl>

class QListView;
class QPainter;
class KPushButton;


namespace Danbooru {

    /**
     * @brief Specific delegate for Danbooru items.
     *
     */

    class DanbooruPost;

    class DanbooruPostDelegate : public QStyledItemDelegate
    {

    Q_OBJECT

    public:
       DanbooruPostDelegate(QListView* itemView);

       void paint(QPainter* painter, const QStyleOptionViewItem& option,
                   const QModelIndex& index) const;
       QSize sizeHint(const QStyleOptionViewItem& option,
                       const QModelIndex& index) const;

       QModelIndex hoveredIndex() const;

    private:

        QListView* m_itemView;
        KPushButton* m_downloadButton;
        KPushButton* m_viewButton;

        int m_buttonSize;
        int m_margin;
        int m_spacing;

        static const int MARGIN;

    Q_SIGNALS:

        /**
         * @brief Emitted when the view button is clicked.
         *
         * @param postUrl the URL to the full picture.
         *
         */
        void postViewRequested(KUrl postUrl);


        /**
         * @brief Emitted when the download button is clicked.
         *
         * @param postUrl the URL to the full picture.
         *
         */
        void postDownloadRequested(KUrl postUrl);


    private Q_SLOTS:
        void viewButtonClicked();
        void downloadButtonClicked();



    };

} // namespace Danbooru

#endif // DANBOORUPOSTDELEGATE_H
